// tgMIDI class for doing MIDI I/O
//
// (c) 2002 by Tom Grandgent (tom@grandgent.com), started on 7/5/2002

#include "tgMIDI.h"
#include "tgDBuffer.h"

// Constructor
tgMIDI::tgMIDI(void)
{
	midi_devices = NULL;
	hin = NULL;
	hout = NULL;
	memset(noteVelocities, 0, sizeof(noteVelocities));
	kbInitialized = FALSE;
	hwndApp = NULL;
	inDeviceID = outDeviceID = -1;
	InitializeCriticalSection(&csOut);

	enumDevices();
}

// Destructor
tgMIDI::~tgMIDI(void)
{
	closeMidiIn();
	closeMidiOut();
	DeleteCriticalSection(&csOut);
}

// Enumerates MIDI input and output devices
void tgMIDI::enumDevices(void)
{
	int indevs, outdevs;
	MIDIINCAPS incaps;
	MIDIOUTCAPS outcaps;
	int i, ret;
	midi_device_t *dev, *ld = NULL;

	// Free the device list if it exists already
	while (midi_devices)
	{
		dev = midi_devices;
		if (dev->name)
			free(dev->name);
		free(dev);
		midi_devices = midi_devices->next;
	}

	// Enumerate input devices
	indevs = midiInGetNumDevs();
	for (i = 0; i < indevs; i++)
	{
		ret = midiInGetDevCaps(i, &incaps, sizeof(incaps));
		if (ret != MMSYSERR_NOERROR)
			continue;

		// Set up a device node for this device
		dev = (midi_device_t *) calloc(1, sizeof(midi_device_t));
		dev->deviceID = i;
		dev->name = strdup(incaps.szPname);
		dev->input_device = 1;
		memcpy(&dev->incaps, &incaps, sizeof(MIDIINCAPS));
		dev->next = NULL;
		if (!ld)
			midi_devices = dev;
		else
			ld->next = dev;
		ld = dev;
	}

	// Enumerate output devices
	outdevs = midiOutGetNumDevs();
	for (i = 0; i < outdevs; i++)
	{
		ret = midiOutGetDevCaps(i, &outcaps, sizeof(outcaps));
		if (ret != MMSYSERR_NOERROR)
			continue;

		// Set up a device node for this device
		dev = (midi_device_t *) calloc(1, sizeof(midi_device_t));
		dev->deviceID = i;
		dev->name = strdup(outcaps.szPname);
		memcpy(&dev->outcaps, &outcaps, sizeof(MIDIOUTCAPS));
		dev->next = NULL;
		if (!ld)
			midi_devices = dev;
		else
			ld->next = dev;
		ld = dev;
	}
}

// Initialize the chosen MIDI input device and start listening
void tgMIDI::openMidiIn(int newMidiInID)
{
	closeMidiIn();

	inDeviceID = newMidiInID;
	if (midiInOpen(&hin, inDeviceID, (DWORD) MidiInProc, (DWORD) this, CALLBACK_FUNCTION) != MMSYSERR_NOERROR)
	{
		hin = NULL;
		MessageBox(hwndApp, "Unable to open MIDI-in device", "midiInOpen() failed...", MB_ICONERROR);
	}
	if (hin)
		midiInStart(hin);
}

// Initialize the chosen MIDI output device
void tgMIDI::openMidiOut(int newMidiOutID)
{
	EnterCriticalSection(&csOut);
	if (hout)
	{
		midiOutClose(hout);
		hout = NULL;
	}

	outDeviceID = newMidiOutID;
	if (midiOutOpen(&hout, outDeviceID, NULL, 0, NULL) != MMSYSERR_NOERROR)
	{
		hout = NULL;
		MessageBox(hwndApp, "Unable to open MIDI-out device", "midiOutOpen() failed...", MB_ICONERROR);
	}
	LeaveCriticalSection(&csOut);
}

// Closes the MIDI input device
void tgMIDI::closeMidiIn(void)
{
	if (hin)
	{
		midiInStop(hin);
		midiInClose(hin);
		hin = NULL;
		inDeviceID = -1;
	}
}

// Closes the MIDI output device
void tgMIDI::closeMidiOut(void)
{
	int i = 0;

	EnterCriticalSection(&csOut);
	if (hout)
	{
		allNotesOff(TRUE);
		//midiOutReset(hout);
		while (midiOutClose(hout) != MMSYSERR_NOERROR && i++ < 10)
			Sleep(200);
		if (i == 10)
		{
			MessageBox(hwndApp, "Unable to close MIDI-out device!", "MIDI Error", MB_ICONERROR);
		}
		hout = NULL;
	}
	outDeviceID = -1;
	LeaveCriticalSection(&csOut);
}

// Turns all notes off on a specific channel
void tgMIDI::allNotesOffChannel(int channel, int abrupt)
{
	// Turn off notes on the MIDI-out device
	if (abrupt)
		sendShortMessage(MAKELONG(MAKEWORD(0xB0 + channel, 120), MAKEWORD(0, 0)));	// abrupt
	else
		sendShortMessage(MAKELONG(MAKEWORD(0xB0 + channel, 123), MAKEWORD(0, 0)));	// non-abrupt
	sendShortMessage(MAKELONG(MAKEWORD(0xB0 + channel, 121), MAKEWORD(0, 0)));	// no effect?
}

// Turns all notes off on all channels
void tgMIDI::allNotesOff(int abrupt)
{
	memset(noteVelocities, 0, sizeof(noteVelocities));
	for (int c = 0; c < 16; c++)
		allNotesOffChannel(c, abrupt);
}

// Handles a MIDI input event (called from the global MidiInProc() callback)
void tgMIDI::handleInputEvent(HMIDIIN hMidiIn, UINT wMsg, DWORD dwParam1, DWORD dwParam2)
{
	static char buf[256];
	unsigned char status, byte1, byte2;
	unsigned char event, channel;

	switch (wMsg)
	{
		case MIM_OPEN:
//			printf("MIDI input opened\n");
			break;
		case MIM_CLOSE:
//			printf("MIDI input closed\n");
			break;
		case MIM_DATA:
			//printf("Data: %d  Timestamp: %d\n", dwParam1, dwParam2);
			byte1 = HIBYTE(LOWORD(dwParam1));
			byte2 = LOBYTE(HIWORD(dwParam1));
			status = LOBYTE(LOWORD(dwParam1));
			event = HINYBBLE(status);
			channel = LONYBBLE(status);
			switch (event)
			{
				case 9:		// Note on
					//printf("Note %s - note %d, velocity %d\n", byte2 ? "on" : "off", byte1, byte2);
					//note_on(TRUE, byte1, byte2, channel);
					break;
				case 0x0B:	// Controller change
					/*ms.channels[channel].last_controller = byte1;
					ms.channels[channel].last_controller_value = byte2;
					if (byte1 == 0)
						ms.channels[channel].last_bank = byte2;
					sendShortMessage(dwParam1);*/
					break;
				case 0x0C:	// Program change
					/*ms.channels[channel].last_program = byte1;
					sendShortMessage(dwParam1);*/
					break;
				default:;
					//OutputDebugString("Sending data\n");
					//sendShortMessage(dwParam1);
			}
//			printf("Channel: %d  Event: %d  Byte1: %d  Byte2: %d\n", channel, event, byte1, byte2);
//			send_midi_data(dwParam1);
			break;
		default:;
//			printf("Unknown MIDI IN message: %d - %d - %d\n", wMsg, dwParam1, dwParam2);
			//midiOutShortMsg(dwParam1);
//			send_midi_data(dwParam1);
	}
}

// Global MidiInProc() callback
void CALLBACK MidiInProc(HMIDIIN hMidiIn, UINT wMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2)
{
	// Use dwInstance to call the correct object's input handling method
	tgMIDI *m = (tgMIDI *) dwInstance;
	m->handleInputEvent(hMidiIn, wMsg, dwParam1, dwParam2);
}

// Send a "note on" event to the MIDI out device
void tgMIDI::noteOn(unsigned char note, unsigned char velocity, unsigned char channel)
{
	noteVelocities[note] = velocity;
	DWORD dwParam1 = MAKELONG(MAKEWORD(MAKEBYTE(channel, 9), note), MAKEWORD(velocity, 0));
	sendShortMessage(dwParam1);
}

// Send a "note off" event to the MIDI out device
void tgMIDI::noteOff(unsigned char note, unsigned char channel)
{
	noteVelocities[note] = 0;
	DWORD dwParam1 = MAKELONG(MAKEWORD(MAKEBYTE(channel, 8), note), MAKEWORD(100, 0));
	sendShortMessage(dwParam1);
}

// Send a "program change" event to the MIDI out device
void tgMIDI::programChange(unsigned char program, unsigned char channel)
{
	_ASSERTE(channel < 16);
	_ASSERTE(program < 128);

	sendShortMessage(MAKELONG(MAKEWORD(MAKEBYTE(channel, 0x0C), program), MAKEWORD(0, 0)));
}

// Send a "control change" event to the MIDI out device
void tgMIDI::controlChange(unsigned char controller, unsigned char value, unsigned char channel)
{
	sendShortMessage(MAKELONG(MAKEWORD(MAKEBYTE(channel, 0x0B), controller), MAKEWORD(value, 0)));
}

// Send a "pitch bend" event to the MIDI out device
void tgMIDI::pitchBend(short int value, unsigned char channel)
{
	unsigned char d1, d2;
	d1 = value & 0x7F;
	d2 = (unsigned char) (value >> 7);
	midiOutShortMsg(hout, MAKELONG(MAKEWORD(MAKEBYTE(channel, 0x0E), d1), MAKEWORD(d2, 0)));
}

// Send a short MIDI message to the MIDI out device (thread-safe)
void tgMIDI::sendShortMessage(DWORD dwMsg)
{
	EnterCriticalSection(&csOut);
	if (hout)
		midiOutShortMsg(hout, dwMsg);
	LeaveCriticalSection(&csOut);
}

// Dump the given file directly to the MIDI-out device
int tgMIDI::sysexDump(FILE *fp)
{
	unsigned char *buf;
	int len;

	// Read the file into a buffer
	len = _filelength(_fileno(fp));
	if (!len)
	{
		// Sysex file is empty
		return 1;
	}
	buf = (unsigned char *) malloc(len + 1);
	fread(buf, len, 1, fp);
	fclose(fp);

	// Dump the data
	sysexDump(buf, len);

	// Free the buffer
	free(buf);

	return 0;
}

// Dump the given data directly to the MIDI-out device
int tgMIDI::sysexDump(unsigned char *buf, int len)
{
	MIDIHDR mh;
	int i, iOpened = FALSE;
	//char text[MAX_PATH];
	//double startTime, endTime, timelen, speed;

	// Open the MIDI-out device
	if (!hout)
	{
		openMidiOut(outDeviceID);
		if (!hout)
		{
			// Unable to open MIDI out device
			return 1;
		}
		iOpened = TRUE;
	}

	EnterCriticalSection(&csOut);

	// Reset the MIDI-out device (is this really necessary?)
	//midiOutReset(hout);

	// Prepare the MIDI out header
	memset(&mh, 0, sizeof(mh));
	mh.lpData = (char *) buf;
	mh.dwBufferLength = len;
	mh.dwBytesRecorded = len;
	// Prepare the sysex buffer for output
	//strcpy(text, "Preparing MIDI out header...");
	//SetWindowText(hwndStatusBar, text);
	midiOutPrepareHeader(hout, &mh, sizeof(mh));

	// Send the sysex buffer!
	//sprintf(text, "Sending %s (%d bytes)...", ms.filename, filelen);
	//SetWindowText(hwndStatusBar, text);
	//startTime = GetTickCount();
	midiOutLongMsg(hout, &mh, sizeof(mh));
	//endTime = GetTickCount();

	// Unprepare the sysex buffer
	do
	{
		i = midiOutUnprepareHeader(hout, &mh, sizeof(mh));
		if (i == MIDIERR_STILLPLAYING)
			Sleep(50);
	} while (i == MIDIERR_STILLPLAYING);

	LeaveCriticalSection(&csOut);

	/*timelen = (endTime - startTime) / 1000.0f;
	if (timelen == 0.0f)
		speed = 0.0f;
	else
		speed = (double) filelen / timelen;
	sprintf(text, "Finished sending %s (%d bytes) in %.1f seconds (%.0f bytes per second)", 
		extract_filename(ms.filename), filelen, timelen, speed);*/

	// Close the MIDI-out device if I opened it
	if (iOpened)
		closeMidiOut();

	return 0;
}

// Given a note number (like 48), returns the name of that note (like "C3")
char *tgMIDI::noteNumToName(int noteNum)
{
	static char *noteNames[] = { "C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B" };
	static char name[8];
	char oct[4];

	strcpy(name, noteNames[noteNum % 12]);
	itoa((noteNum - 12) / 12, oct, 10);
	strcat(name, oct);
	
	return name;
}

// Given a note number (like 48), returns the name of that note without an octave number (like "C")
char *tgMIDI::noteNumToNameWithoutOctave(int noteNum)
{
	static char *noteNames[] = { "C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B" };
	return noteNames[noteNum % 12];
}

// Given a note name (like "C3"), returns the number of that note (like 48)
int tgMIDI::noteNameToNum(char *name)
{
	static char *noteNames[] = { "C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B" };

	_ASSERTE(name);
	_ASSERTE(strlen(name) >= 2);

	// *** Get octave ***
	// Start at the end
	char *ch = name + strlen(name) - 1;
	// Seek backwards until a non-digit is found
	while ((ch > name) && (*(ch - 1) >= '0') && (*(ch - 1) <= '9'))
		ch--;
	// ch now points to the first digit of the octave
	int oct = atoi(ch);

	// *** Get note within the octave ***
	for (int i = 0; i < 12; i++)
		if (!strnicmp(name, noteNames[i], ch - name))
			return (oct + 1) * 12 + i;

	// Oops.. couldn't figure out which note it is
	return 0;
}

// Returns 1 if the given note is a black key on the keyboard
int tgMIDI::isBlackKey(int noteNum)
{
	noteNum = noteNum % 12;
	if (noteNum == 1 || noteNum == 3 || noteNum == 6 || noteNum == 8 || noteNum == 10)
		return 1;
	
	return 0;
}

// Sets parameters for keyboard-drawing
// All arguments are optional (if an argument is not specified, previous value is assumed)
// Pass NULL or -1 to avoid specifying an argument
void tgMIDI::setKeyboardParameters(HWND hwnd, RECT *r, int lowNote, int highNote)
{
	kbInitialized = TRUE;
	if (hwnd)
		kbHwnd = hwnd;
	if (r)
		memcpy(&kbRect, r, sizeof(RECT));
	if (lowNote != -1)
		kbLowNote = lowNote;
	if (highNote != -1)
	{
		kbHighNote = highNote;
		if (kbHighNote > 127)
			kbHighNote = 127;
	}
}

int tgMIDI::pointToNote(int mx, int my)
{
	return traverseKeyboard(NULL, 0, FALSE, mx, my);
}

int tgMIDI::traverseKeyboard(unsigned char *noteArray, int displayFlags, int draw, int mx, int my)
{
	unsigned char *notes = noteArray;
	char *str;
	int i, bx, lastRight, ix;
	HBRUSH hibrush;
	HFONT hFont;
	HDC hdc;
	HPEN pen = NULL;
	float whiteKeyWidth, blackKeyWidth, whiteKeyHeight, blackKeyHeight, x;
	int processedBlackKeys = FALSE;

	if (!notes)
		notes = noteVelocities;

#ifdef TGDBUFFER_DEFINED
		static tgDBuffer db(kbHwnd);
#endif

	// Get a device context to draw to
	if (draw)
	{
#ifdef TGDBUFFER_DEFINED
		hdc = db.getBuffer(NULL);
#else
		hdc = GetDC(kbHwnd);
#endif

		// Make a "highlighted key" brush
		if (displayFlags & DK_NOTES)
		{
			hibrush = CreateSolidBrush(RGB(0, 0, 255));
			SelectObject(hdc, GetStockObject(BLACK_PEN));
		}
		else
		{
			pen = CreatePen(PS_SOLID, 0, RGB(96, 96, 96));
			SelectObject(hdc, pen);
		}

		// Make a font for showing the note names
		if (displayFlags & DK_NOTENAMES)
		{
			hFont = CreateFont(12, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, 
				OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH, "Arial");
			SelectObject(hdc, hFont);
			SetTextAlign(hdc, TA_CENTER);
			SetTextColor(hdc, RGB(255, 255, 255));
			SetBkMode(hdc, TRANSPARENT);
		}
	}

	// Count # of white keys and # of black keys
	int numwhitekeys = 0, numblackkeys = 0;
	for (i = kbLowNote; i <= kbHighNote; i++)
		if (isBlackKey(i))
			numblackkeys++;
		else
			numwhitekeys++;

	// Figure out how wide and tall the keys should be
	_ASSERTE(numwhitekeys > 0);
	whiteKeyWidth = (float) (kbRect.right - kbRect.left) / (float) numwhitekeys;
	blackKeyWidth = whiteKeyWidth * 0.666f;
	whiteKeyHeight = (float) (kbRect.bottom - kbRect.top);
	blackKeyHeight = whiteKeyHeight * 0.65f;

	// Draw a blank white rectangle over the whole space first
	if (draw)
	{
		SelectObject(hdc, GetStockObject(WHITE_BRUSH));
		Rectangle(hdc, kbRect.left, kbRect.top, kbRect.right, kbRect.bottom);
	}

	// If we're not drawing, we need to check the black keys first :(
	if (!draw && !processedBlackKeys)
		goto ProcessBlackKeys;

ProcessWhiteKeys:
	// Draw the white keys
	x = 0.0f;
	lastRight = 0;
	for (i = kbLowNote; i <= kbHighNote; i++)
	{
		// We're only concerned with white keys at this point
		if (!isBlackKey(i))
		{
			// If highlighting notes and this is a note to highlight, select highlight brush
			if (draw && (displayFlags & DK_NOTES) && notes[i])
				SelectObject(hdc, hibrush);

			// Draw the white key
			// This ix stuff is to compensate for nasty pixel jumps due to lack of precision :(
			ix = (int) x;
			if (ix > lastRight)
				ix = lastRight;
			if (draw)
				Rectangle(hdc, ix, kbRect.top, (int) (x + whiteKeyWidth), kbRect.bottom);
			else
				if (mx >= ix && mx <= (int) (x + whiteKeyWidth) && 
					my >= kbRect.top && my <= kbRect.bottom)
					return i;
			lastRight = (int) (x + whiteKeyWidth);
			
			// If showing note names for highlighted notes and this is a note to highlight, 
			// draw the note name text
			if (draw && (displayFlags & DK_NOTENAMES) && notes[i])
			{
				str = noteNumToNameWithoutOctave(i);
				TextOut(hdc, (int) (x + whiteKeyWidth / 2.0f), 
					kbRect.top + (int) (whiteKeyHeight * 0.8f), 
					str, strlen(str));
			}
			
			// Bring back the white brush if we highlighted this note
			if (draw && (displayFlags & DK_NOTES) && notes[i])
				SelectObject(hdc, GetStockObject(WHITE_BRUSH));

			// Move on to the next white key
			x += whiteKeyWidth;
		}
	}
	if (!draw && processedBlackKeys)
		goto DoneProcessingKeys;

ProcessBlackKeys:
	// Draw the black keys
	if (draw)
	{
		if (displayFlags & DK_NOTES)
			SelectObject(hdc, GetStockObject(BLACK_BRUSH));
		else
			SelectObject(hdc, GetStockObject(LTGRAY_BRUSH));
	}
	x = 0.0f;
	for (i = kbLowNote; i <= kbHighNote; i++)
	{
		// We're only concerned with black keys at this point
		if (isBlackKey(i))
		{
			// Figure out spacing of the black key depending on which note it is
			if (i % 12 == 1 || i % 12 == 6)
				bx = (int) (x - blackKeyWidth * 0.6f);
			else if (i % 12 == 3 || i % 12 == 10)
				bx = (int) (x - blackKeyWidth * 0.4f);
			else
				bx = (int) (x - blackKeyWidth * 0.5f);

			// If highlighting notes and this is a note to highlight, select highlight brush
			if (draw && (displayFlags & DK_NOTES) && notes[i])
				SelectObject(hdc, hibrush);

			// Draw the black key
			if (draw)
				Rectangle(hdc, bx, kbRect.top, bx + (int) blackKeyWidth, 
					kbRect.top + (int) blackKeyHeight);
			else
				if (mx >= bx && mx <= bx + (int) blackKeyWidth && 
					my >= kbRect.top && my <= kbRect.top + (int) blackKeyHeight)
					return i;

			// If showing note names for highlighted notes and this is a note to highlight, 
			// draw the note name text
			if (draw && (displayFlags & DK_NOTENAMES) && notes[i])
			{
				str = noteNumToNameWithoutOctave(i);
				TextOut(hdc, (int) (bx + blackKeyWidth / 2.0f), 
					kbRect.top + (int) (blackKeyHeight * 0.7f), 
					str, strlen(str));
			}
			
			// Bring back the black brush if we highlighted this note
			if (draw && (displayFlags & DK_NOTES) && notes[i])
				SelectObject(hdc, GetStockObject(BLACK_BRUSH));
		}
		else
			x += whiteKeyWidth;
	}

	if (!draw && !processedBlackKeys)
	{
		processedBlackKeys = TRUE;
		goto ProcessWhiteKeys;
	}
DoneProcessingKeys:

	if (draw)
	{
#ifdef TGDBUFFER_DEFINED
		db.draw();
#else
		ReleaseDC(kbHwnd, hdc);
#endif
		if (displayFlags & DK_NOTES)
			DeleteObject(hibrush);

		if (displayFlags & DK_NOTENAMES)
			DeleteObject(hFont);

		if (pen)
			DeleteObject(pen);
	}
	else
		return -1;

	return 0;
}

// Draws a keyboard showing which notes are pressed
void tgMIDI::drawKeyboard(unsigned char *noteArray, int displayFlags)
{
	traverseKeyboard(noteArray, displayFlags, TRUE, 0, 0);
}

// Returns the current velocity of the given note (0 means it's not sounding)
unsigned char tgMIDI::getNoteVelocity(unsigned char note)
{
	_ASSERTE(note < 128);

	return noteVelocities[note];
}

///////// GLOBAL DATA FOLLOWS:

char *gmProgramNames[128] = {

"Acoustic Grand Piano",
"Bright Acoustic Piano",
"Electric Grand Piano",
"Honky-tonk Piano",
"Electric Piano 1",
"Electric Piano 2",
"Harpsichord",
"Clavi",
"Celesta",
"Glockenspiel",
"Music Box",
"Vibraphone",
"Marimba",
"Xylophone",
"Tubular Bells",
"Dulcimer",
"Drawbar Organ",
"Percussive Organ",
"Rock Organ",
"Church Organ",
"Reed Organ",
"Accordion",
"Harmonica",
"Tango Accordion",
"Acoustic Guitar (nylon)",
"Acoustic Guitar (steel)",
"Electric Guitar (jazz)",
"Electric Guitar (clean)",
"Electric Guitar (muted)",
"Overdriven Guitar",
"Distortion Guitar",
"Guitar harmonics",
"Acoustic Bass",
"Electric Bass (finger)",
"Electric Bass (pick)",
"Fretless Bass",
"Slap Bass 1",
"Slap Bass 2",
"Synth Bass 1",
"Synth Bass 2",
"Violin",
"Viola",
"Cello",
"Contrabass",
"Tremolo Strings",
"Pizzicato Strings",
"Orchestral Harp",
"Timpani",
"String Ensemble 1",
"String Ensemble 2",
"SynthStrings 1",
"SynthStrings 2",
"Choir Aahs",
"Voice Oohs",
"Synth Voice",
"Orchestra Hit",
"Trumpet",
"Trombone",
"Tuba",
"Muted Trumpet",
"French Horn",
"Brass Section",
"SynthBrass 1",
"SynthBrass 2",
"Soprano Sax",
"Alto Sax",
"Tenor Sax",
"Baritone Sax",
"Oboe",
"English Horn",
"Bassoon",
"Clarinet",
"Piccolo",
"Flute",
"Recorder",
"Pan Flute",
"Blown Bottle",
"Shakuhachi",
"Whistle",
"Ocarina",
"Lead 1 (square)",
"Lead 2 (sawtooth)",
"Lead 3 (calliope)",
"Lead 4 (chiff)",
"Lead 5 (charang)",
"Lead 6 (voice)",
"Lead 7 (fifths)",
"Lead 8 (bass + lead)",
"Pad 1 (new age)",
"Pad 2 (warm)",
"Pad 3 (polysynth)",
"Pad 4 (choir)",
"Pad 5 (bowed)",
"Pad 6 (metallic)",
"Pad 7 (halo)",
"Pad 8 (sweep)",
"FX 1 (rain)",
"FX 2 (soundtrack)",
"FX 3 (crystal)",
"FX 4 (atmosphere)",
"FX 5 (brightness)",
"FX 6 (goblins)",
"FX 7 (echoes)",
"FX 8 (sci-fi)",
"Sitar",
"Banjo",
"Shamisen",
"Koto",
"Kalimba",
"Bag pipe",
"Fiddle",
"Shanai",
"Tinkle Bell",
"Agogo",
"Steel Drums",
"Woodblock",
"Taiko Drum",
"Melodic Tom",
"Synth Drum",
"Reverse Cymbal",
"Guitar Fret Noise",
"Breath Noise",
"Seashore",
"Bird Tweet",
"Telephone Ring",
"Helicopter",
"Applause",
"Gunshot"

};
